@@ -1,27 +1,51 @@
 try:
-    import simplejson as json # Prefer this as it is faster
-except ImportError: # pragma: no cover
+    import simplejson as json  # Prefer this as it is faster
+except ImportError:  # pragma: no cover
     try:
         import json
     except ImportError:
         raise ImportError("Could not find any JSON module to import - "
-            + "please install simplejson or jsonlib to continue")
+                          + "please install simplejson or jsonlib to continue")
 
 import urllib
-import httplib
 import re
 import copy
 import base64
-from urlparse import urlparse
+import sys
+import logging
 from itertools import groupby
-import UserDict
+from contextlib import closing
+
+P3K = sys.version_info >= (3, 0)
+
+logging.basicConfig()
+
+try:
+    # Python 2.x imports
+    from UserDict import UserDict
+    from urllib import urlencode
+    from urllib2 import urlopen
+    from urllib2 import HTTPError
+    from urllib2 import Request
+    from urlparse import urlparse
+    import httplib
+except ImportError:
+    # Python 3.x imports
+    from urllib.parse import urlencode
+    from urllib.parse import urlparse
+    from urllib.request import urlopen
+    from urllib.request import Request
+    from urllib.error import HTTPError
+    from collections import UserDict
+    import http.client as httplib
 
 from intermine.errors import WebserviceError
 from intermine.model import Attribute, Reference, Collection
 
-USER_AGENT = 'WebserviceInterMinePythonAPIClient'
+from intermine import VERSION
 
-class EnrichmentLine(UserDict.UserDict):
+
+class EnrichmentLine(UserDict):
     """
     An object that represents a result returned from the enrichment service.
     ========================================================================
@@ -39,10 +63,11 @@ class EnrichmentLine(UserDict.UserDict):
     def __getattr__(self, name):
         if name is not None:
             key_name = name.replace('_', '-')
-            if key_name in self.keys():
+            if key_name in list(self.keys()):
                 return self.data[key_name]
         raise AttributeError(name)
 
+
 class ResultObject(object):
     """
     An object used to represent result records as returned in jsonobjects format
@@ -67,18 +92,25 @@ class ResultObject(object):
         self.selected_attributes = [v for v in stripped if "." not in v]
         self.reference_paths = dict(((k, list(i)) for k, i in groupby(stripped, lambda x: x[:x.find(".") + 1])))
         self._data = data
-        self._cld = cld if "class" not in data or cld.name == data["class"] else cld.model.get_class(data["class"])
+        # Make sure this object has the most specific class desc. possible
+        class_name = data['class']
+        if "class" not in data or cld.name == class_name:
+            self._cld = cld
+        else:  # this could be a composed class - behave accordingly.
+            self._cld = cld.model.get_class(class_name)
+
         self._attr_cache = {}
 
     def __str__(self):
         dont_show = set(["objectId", "class"])
-        return "%s(%s)" % (self._cld.name, ",  ".join("%s = %r" % (k, v) for k, v in self._data.items()
-            if not isinstance(v, dict) and not isinstance(v, list) and k not in dont_show))
+        return "%s(%s)" % (self._cld.name, ",  ".join("%s = %r" % (k, v) for k, v in list(self._data.items())
+                                                      if not isinstance(v, dict) and not isinstance(v,
+                                                                                                    list) and k not in dont_show))
 
     def __repr__(self):
         dont_show = set(["objectId", "class"])
-        return "%s(%s)" % (self._cld.name, ", ".join("%s = %r" % (k, getattr(self, k)) for k in self._data.keys()
-            if k not in dont_show))
+        return "%s(%s)" % (self._cld.name, ", ".join("%s = %r" % (k, getattr(self, k)) for k in list(self._data.keys())
+                                                     if k not in dont_show))
 
     def __getattr__(self, name):
         if name in self._attr_cache:
@@ -104,7 +136,7 @@ class ResultObject(object):
                 if data is None:
                     attr = []
                 else:
-                    attr = map(lambda x: ResultObject(x, fld.type_class, ref_paths), data)
+                    attr = [ResultObject(x, fld.type_class, ref_paths) for x in data]
             else:
                 if data is None:
                     attr = None
@@ -128,24 +160,25 @@ class ResultObject(object):
 
     def _fetch_attr(self, fld):
         if fld.name in self.selected_attributes:
-            return None # Was originally selected - no point asking twice
+            return None  # Was originally selected - no point asking twice
         c = self._cld
         if "id" not in c:
-            return None # Cannot reliably fetch anything without access to the objectId.
-        q = c.model.service.query(c, fld).where(id = self.id)
+            return None  # Cannot reliably fetch anything without access to the objectId.
+        q = c.model.service.query(c, fld).where(id=self.id)
         r = q.first()
         return r._data[fld.name] if fld.name in r._data else None
 
     def _fetch_reference(self, ref):
         if ref.name + "." in self.reference_paths:
-            return None # Was originally selected - no point asking twice.
+            return None  # Was originally selected - no point asking twice.
         c = self._cld
         if "id" not in c:
-            return None # Cannot reliably fetch anything without access to the objectId.
-        q = c.model.service.query(ref).outerjoin(ref).where(id = self.id)
+            return None  # Cannot reliably fetch anything without access to the objectId.
+        q = c.model.service.query(ref).outerjoin(ref).where(id=self.id)
         r = q.first()
         return r._data[ref.name] if ref.name in r._data else None
 
+
 class ResultRow(object):
     """
     An object for representing a row of data received back from the server.
@@ -189,9 +222,9 @@ class ResultRow(object):
         root = re.sub("\..*$", "", self.views[0])
         parts = [root + ":"]
         for view in self.views:
-           short_form = re.sub("^[^.]+.", "", view)
-           value = self[view]
-           parts.append(short_form + "=" + repr(value))
+            short_form = re.sub("^[^.]+.", "", view)
+            value = self[view]
+            parts.append(short_form + "=" + repr(value))
         return " ".join(parts)
 
     def __call__(self, name):
@@ -210,14 +243,9 @@ class ResultRow(object):
         """Return a list view of this row"""
         return [x for x in self.data]
 
-
     def to_d(self):
         """Return a dictionary view of this row"""
-        d = {}
-        for view in self.views:
-            d[view] = self[view]
-
-        return d
+        return dict(list(self.items()))
 
     def items(self):
         return [(view, self[view]) for view in self.views]
@@ -243,7 +271,8 @@ class ResultRow(object):
             self._get_index_for(key)
             return True
         except KeyError:
-           return False
+            return False
+
 
 class TableResultRow(ResultRow):
     """
@@ -254,30 +283,27 @@ class TableResultRow(ResultRow):
         if isinstance(key, int):
             return self.data[key]["value"]
         elif isinstance(key, slice):
-            vals = map(lambda x: x["value"], self.data[key])
-            return vals
+            return [x["value"] for x in self.data[key]]
         else:
             index = self._get_index_for(key)
             return self.data[index]["value"]
 
     def to_l(self):
         """Return a list view of this row"""
-        return map(lambda x: x["value"], self.data)
+        return [x["value"] for x in self.data]
+
+
+def encode_str(s):
+    return s.encode('utf8') if hasattr(s, 'encode') else s
+
+
+def decode_binary(b):
+    return b.decode('utf8') if hasattr(b, 'decode') else b
+
 
 def encode_dict(input_d):
-    output = {}
-    for k, v in input_d.iteritems():
-        if isinstance(k, unicode):
-            k = k.encode('utf8')
-        elif isinstance(k, str):
-            k.decode('utf8')
+    return dict((encode_str(k), encode_str(v)) for k, v in list(input_d.items()))
 
-        if isinstance(v, unicode):
-            v = v.encode('utf8')
-        elif isinstance(v, str):
-            v.decode('utf8')
-        output[k] = v
-    return output
 
 class ResultIterator(object):
     """
@@ -304,7 +330,7 @@ class ResultIterator(object):
         Services are responsible for getting result iterators. You will
         not need to create one manually.
 
-        @param root: The root path (eg: "http://www.flymine.org/query/service")
+        @param root: The root path (eg: "https://www.flymine.org/query/service")
         @type root: string
         @param path: The resource path (eg: "/query/results")
         @type path: string
@@ -320,11 +346,11 @@ class ResultIterator(object):
         @raise ValueError: if the row format is incorrect
         @raise WebserviceError: if the request is unsuccessful
         """
-        if rowformat.startswith("object"): # Accept "object", "objects", "objectformat", etc...
-            rowformat = "jsonobjects" # these are synonymous
+        if rowformat.startswith("object"):  # Accept "object", "objects", "objectformat", etc...
+            rowformat = "jsonobjects"  # these are synonymous
         if rowformat not in self.ROW_FORMATS:
             raise ValueError("'%s' is not one of the valid row formats (%s)"
-                    % (rowformat, repr(list(self.ROW_FORMATS))))
+                             % (rowformat, repr(list(self.ROW_FORMATS))))
 
         self.row = ResultRow if service.version >= 8 else TableResultRow
 
@@ -332,14 +358,14 @@ class ResultIterator(object):
             if service.version >= 8:
                 params.update({"format": "json"})
             else:
-                params.update({"format" : "jsonrows"})
+                params.update({"format": "jsonrows"})
         elif rowformat == 'tsv':
             params.update({"format": "tab"})
         else:
-            params.update({"format" : rowformat})
+            params.update({"format": rowformat})
 
-        self.url  = service.root + path
-        self.data = urllib.urlencode(encode_dict(params), True)
+        self.url = service.root + path
+        self.data = urlencode(encode_dict(params), True)
         self.view = view
         self.opener = service.opener
         self.cld = cld
@@ -351,7 +377,8 @@ class ResultIterator(object):
         Return the number of items in this iterator
         ===========================================
 
-        Note that this requires iterating over the full result set.
+        Note that this requires iterating over the full result set, making the
+        request in the process.
         """
         c = 0
         for x in self:
@@ -372,20 +399,24 @@ class ResultIterator(object):
 
         try:
             reader = {
-                "tsv"         : flat_file_parser,
-                "csv"         : flat_file_parser,
-                "count"       : flat_file_parser,
-                "json"        : simple_json_parser,
-                "jsonrows"    : simple_json_parser,
-                "list"        : lambda: JSONIterator(con, lambda x: self.row(x, self.view).to_l()),
-                "rr"          : lambda: JSONIterator(con, lambda x: self.row(x, self.view)),
-                "dict"        : lambda: JSONIterator(con, lambda x: self.row(x, self.view).to_d()),
-                "jsonobjects" : lambda: JSONIterator(con, lambda x: ResultObject(x, self.cld, self.view))
+                "tsv": flat_file_parser,
+                "csv": flat_file_parser,
+                "count": flat_file_parser,
+                "json": simple_json_parser,
+                "jsonrows": simple_json_parser,
+                "list": lambda: JSONIterator(con, lambda x: self.row(x, self.view).to_l()),
+                "rr": lambda: JSONIterator(con, lambda x: self.row(x, self.view)),
+                "dict": lambda: JSONIterator(con, lambda x: self.row(x, self.view).to_d()),
+                "jsonobjects": lambda: JSONIterator(con, lambda x: ResultObject(x, self.cld, self.view))
             }.get(self.rowformat)()
-        except Exception, e:
-            raise Exception("Couldn't get iterator for "  + self.rowformat + str(e))
+        except Exception as e:
+            raise Exception("Couldn't get iterator for " + self.rowformat)
         return reader
 
+    def __next__(self):
+        """2.x to 3.x bridge"""
+        return self.next()
+
     def next(self):
         """
         Returns the next row, in the appropriate format
@@ -400,6 +431,7 @@ class ResultIterator(object):
             self._it = None
             raise StopIteration
 
+
 class FlatFileIterator(object):
     """
     An iterator for handling results returned as a flat file (TSV/CSV).
@@ -424,13 +456,18 @@ class FlatFileIterator(object):
     def __iter__(self):
         return self
 
+    def __next__(self):
+        """2.x to 3.x bridge"""
+        return self.next()
+
     def next(self):
         """Return a parsed line of data"""
-        line = self.connection.next().strip()
+        line = decode_binary(next(self.connection)).strip()
         if line.startswith("[ERROR]"):
             raise WebserviceError(line)
         return self.parser(line)
 
+
 class JSONIterator(object):
     """
     An iterator for handling results returned in the JSONRows format
@@ -439,6 +476,8 @@ class JSONIterator(object):
     This iterator can be used as the sub iterator in a ResultIterator
     """
 
+    LOG = logging.getLogger('JSONIterator')
+
     def __init__(self, connection, parser):
         """
         Constructor
@@ -459,6 +498,10 @@ class JSONIterator(object):
     def __iter__(self):
         return self
 
+    def __next__(self):
+        """2.6.x-3.x bridge"""
+        return self.next()
+
     def next(self):
         """Returns a parsed row of data"""
         if self._is_finished:
@@ -467,8 +510,9 @@ class JSONIterator(object):
 
     def parse_header(self):
         """Reads out the header information from the connection"""
+        self.LOG.debug('Connection = {0}'.format(self.connection))
         try:
-            line = self.connection.next().strip()
+            line = decode_binary(next(self.connection)).strip()
             self.header += line
             if not line.endswith('"results":['):
                 self.parse_header()
@@ -505,7 +549,7 @@ class JSONIterator(object):
         """
         next_row = None
         try:
-            line = self.connection.next()
+            line = decode_binary(next(self.connection))
             if line.startswith("]"):
                 self.footer += line;
                 for otherline in self.connection:
@@ -516,9 +560,9 @@ class JSONIterator(object):
                 if len(line) > 0:
                     try:
                         row = json.loads(line)
-                    except json.decoder.JSONDecodeError, e:
+                    except json.decoder.JSONDecodeError as e:
                         raise WebserviceError("Error parsing line from results: '"
-                                + line + "' - " + str(e))
+                                              + line + "' - " + str(e))
                     next_row = self.parser(row)
         except StopIteration:
             raise WebserviceError("Connection interrupted")
@@ -529,19 +573,21 @@ class JSONIterator(object):
         else:
             return next_row
 
+
 def encode_headers(headers):
     return dict((k.encode('ascii') if isinstance(k, unicode) else k, \
                  v.encode('ascii') if isinstance(v, unicode) else v) \
-                 for k, v in headers.items())
+                for k, v in list(headers.items()))
 
-class InterMineURLOpener(urllib.FancyURLopener):
+
+class InterMineURLOpener(object):
     """
-    Specific implementation of urllib.FancyURLOpener for this client
+    Specific implementation of FancyURLopener for this client
     ================================================================
 
     Provides user agent and authentication headers, and handling of errors
     """
-    version = "InterMine-Python-Client-1.07.01"
+    USER_AGENT = "InterMine-Client-{0}/python-{1}".format(VERSION, sys.version_info)
     PLAIN_TEXT = "text/plain"
     JSON = "application/json"
 
@@ -554,21 +600,15 @@ class InterMineURLOpener(urllib.FancyURLopener):
 
         Return a new url-opener with the appropriate credentials
         """
-        urllib.FancyURLopener.__init__(self)
         self.token = token
-        self.plain_post_header = {
-            "Content-Type": "text/plain; charset=utf-8",
-            "UserAgent": USER_AGENT
-        }
         if credentials and len(credentials) == 2:
-            base64string = 'Basic ' + base64.encodestring('%s:%s' % credentials)[:-1]
-            self.addheader("Authorization", base64string)
-            self.plain_post_header["Authorization"] = base64string
+            encoded = '{0}:{1}'.format(*credentials).encode('utf8')
+            base64string = 'Basic {0}'.format(base64.encodestring(encoded)[:-1].decode('ascii'))
+            self.auth_header = base64string
             self.using_authentication = True
         elif self.token is not None:
-            token_header = 'Token {}'.format(self.token)
-            self.addheader('Authorization', token_header)
-            self.plain_post_header['Authorization'] = token_header
+            token_header = 'Token {0}'.format(self.token)
+            self.auth_header = token_header
             self.using_authentication = True
         else:
             self.using_authentication = False
@@ -576,45 +616,63 @@ class InterMineURLOpener(urllib.FancyURLopener):
     def clone(self):
         clone = InterMineURLOpener()
         clone.token = self.token
-        clone.plain_post_header = dict(self.plain_post_header.items())
         clone.using_authentication = self.using_authentication
         if self.using_authentication:
-            clone.addheader('Authorization', self.plain_post_header['Authorization'])
+            clone.auth_header = self.auth_header
         return clone
 
+    def headers(self, content_type=None, accept=None):
+        h = {'UserAgent': self.USER_AGENT}
+        if self.using_authentication:
+            h['Authorization'] = self.auth_header
+        if content_type is not None:
+            h['Content-Type'] = content_type
+        if accept is not None:
+            h['Accept'] = accept
+        return h
+
     def post_plain_text(self, url, body):
-        if isinstance(body, unicode):
-            body = body.encode('utf8')
-        elif isinstance(body, str):
-            body.decode('utf8')
         return self.post_content(url, body, InterMineURLOpener.PLAIN_TEXT)
 
-    def post_content(self, url, body, mimetype, charset = "utf-8"):
-        headers = {
-            "Content-Type": "%s; charset=%s" % (mimetype, charset),
-            "UserAgent": USER_AGENT
-        }
-        if self.using_authentication:
-            headers['Authorization'] = self.plain_post_header['Authorization']
+    def post_content(self, url, body, mimetype, charset="utf-8"):
+        content_type = "{0}; charset={1}".format(mimetype, charset)
 
-        url = self.prepare_url(url)
-        o = urlparse(url)
-        con = httplib.HTTPConnection(o.hostname, o.port)
-        con.request('POST', url, body, encode_headers(headers))
-        resp = con.getresponse()
-        content = resp.read()
-        con.close()
-        if resp.status != 200:
-            raise WebserviceError(resp.status, resp.reason, content)
-        return content
+        with closing(self.open(url, body, {'Content-Type': content_type})) as f:
+            return f.read()
 
-    def open(self, url, data=None):
+    def open(self, url, data=None, headers=None, method=None):
         url = self.prepare_url(url)
-        return urllib.FancyURLopener.open(self, url, data)
+        buff = data if data is None else bytearray(data, 'utf8')
+        hs = self.headers()
+        if headers is not None:
+            hs.update(headers)
+        req = Request(url, buff, headers=hs)
+        if method is not None:
+            req.get_method = lambda: method
+        try:
+            return urlopen(req)
+        except HTTPError as e:
+            args = (url, e, e.code,  # The next two lines are python2.6 workarounds
+                    e.reason if hasattr(e, 'reason') else None,
+                    e.headers if hasattr(e, 'headers') else None)
+            handler = {
+                400: self.http_error_400,
+                401: self.http_error_401,
+                403: self.http_error_403,
+                404: self.http_error_404,
+                500: self.http_error_500
+            }.get(e.code, self.http_error_default)
+            handler(*args)
+
+    def read(self, url, data=None):
+        with closing(self.open(url, data)) as conn:
+            content = conn.read()
+            return decode_binary(content)
 
     def prepare_url(self, url):
+        # Generally unnecessary these days - will be deprecated one of these days.
         if self.token:
-            token_param = urllib.urlencode(encode_dict(dict(token = self.token)))
+            token_param = urlencode(encode_dict(dict(token=self.token)))
             o = urlparse(url)
             if o.query:
                 url += "&" + token_param
@@ -624,16 +682,8 @@ class InterMineURLOpener(urllib.FancyURLopener):
         return url
 
     def delete(self, url):
-        url = self.prepare_url(url)
-        o = urlparse(url)
-        con = httplib.HTTPConnection(o.hostname, o.port)
-        con.request('DELETE', url, None, self.plain_post_header)
-        resp = con.getresponse()
-        content = resp.read()
-        con.close()
-        if resp.status != 200:
-            raise WebserviceError(resp.status, resp.reason, content)
-        return content
+        with closing(self.open(url, method='DELETE')) as f:
+            return f.read()
 
     def http_error_default(self, url, fp, errcode, errmsg, headers):
         """Re-implementation of http_error_default, with content now supplied by default"""
@@ -673,7 +723,8 @@ class InterMineURLOpener(urllib.FancyURLopener):
         content = fp.read()
         fp.close()
         if self.using_authentication:
-            raise WebserviceError("Insufficient permissions", errcode, errmsg, content)
+            auth = self.auth_header
+            raise WebserviceError("Insufficient permissions - {0}".format(auth), errcode, errmsg, content)
         else:
             raise WebserviceError("No permissions - not logged in", errcode, errmsg, content)
 
@@ -717,6 +768,7 @@ class InterMineURLOpener(urllib.FancyURLopener):
         except:
             message = content
         raise WebserviceError("Missing resource", errcode, errmsg, message)
+
     def http_error_500(self, url, fp, errcode, errmsg, headers, data=None):
         """
         Handle 500 HTTP errors, attempting to return informative error messages
@@ -735,4 +787,3 @@ class InterMineURLOpener(urllib.FancyURLopener):
         except:
             message = content
         raise WebserviceError("Internal server error", errcode, errmsg, message)
-
