@@ -1,27 +1,29 @@
+from __future__ import unicode_literals
+
 from xml.dom import minidom
-import urllib
-from urlparse import urlparse
-import base64
-import UserDict
 from contextlib import closing
 
-#class UJsonLibDecoder(object): # pragma: no cover
-#    def __init__(self):
-#        self.loads = ujson.decode
-#
-# Use core json for 2.6+, simplejson for <=2.5
-#try:
-#    import ujson
-#    json = UJsonLibDecoder()
-#except ImportError: # pragma: no cover
+import requests
+
 try:
-    import simplejson as json # Prefer this as it is faster
-except ImportError: # pragma: no cover
+    from urlparse import urlparse
+    from UserDict import DictMixin
+    from urllib import urlopen
+    from urllib import urlencode
+except ImportError:
+    from urllib.parse import urlparse
+    from urllib.parse import urlencode
+    from collections import MutableMapping as DictMixin
+    from urllib.request import urlopen
+
+try:
+    import simplejson as json  # Prefer this as it is faster
+except ImportError:  # pragma: no cover
     try:
         import json
     except ImportError:
         raise ImportError("Could not find any JSON module to import - "
-            + "please install simplejson or jsonlib to continue")
+                          + "please install simplejson or jsonlib to continue")
 
 # Local intermine imports
 from intermine.query import Query, Template
@@ -46,7 +48,8 @@ __organization__ = "InterMine"
 __license__ = "LGPL"
 __contact__ = "dev@intermine.org"
 
-class Registry(object, UserDict.DictMixin):
+
+class Registry(DictMixin):
     """
     A Class representing an InterMine registry.
     ===========================================
@@ -83,13 +86,16 @@ class Registry(object, UserDict.DictMixin):
 
     MINES_PATH = "/mines.json"
 
-    def __init__(self, registry_url="http://www.intermine.org/registry"):
+    def __init__(
+            self, registry_url="http://www.intermine.org/registry"):
         self.registry_url = registry_url
         opener = InterMineURLOpener()
         data = opener.open(registry_url + Registry.MINES_PATH).read()
         mine_data = json.loads(data)
-        self.__mine_dict = dict(( (mine["name"], mine) for mine in mine_data["mines"]))
-        self.__synonyms = dict(( (name.lower(), name) for name in self.__mine_dict.keys() ))
+        self.__mine_dict = dict(((mine["name"], mine)
+                                 for mine in mine_data["mines"]))
+        self.__synonyms = dict(((name.lower(), name)
+                                for name in list(self.__mine_dict.keys())))
         self.__mine_cache = {}
 
     def __contains__(self, name):
@@ -99,19 +105,36 @@ class Registry(object, UserDict.DictMixin):
         lc = name.lower()
         if lc in self.__synonyms:
             if lc not in self.__mine_cache:
-                self.__mine_cache[lc] = Service(self.__mine_dict[self.__synonyms[lc]]["webServiceRoot"])
+                self.__mine_cache[lc] = Service(
+                    self.__mine_dict[self.__synonyms[lc]]["webServiceRoot"])
             return self.__mine_cache[lc]
         else:
             raise KeyError("Unknown mine: " + name)
 
     def __setitem__(self, name, item):
-        raise NotImplementedError("You cannot add items to a registry")
+        raise NotImplementedError(
+            "You cannot add items to a registry")
 
     def __delitem__(self, name):
-        raise NotImplementedError("You cannot remove items from a registry")
+        raise NotImplementedError(
+            "You cannot remove items from a registry")
+
+    def __len__(self):
+        return len(self.__mine_dict)
+
+    def __iter__(self):
+        return iter(self.__mine_dict)
 
     def keys(self):
-        return self.__mine_dict.keys()
+        return list(self.__mine_dict.keys())
+
+
+def ensure_str(stringlike):
+    if hasattr(stringlike, 'decode'):
+        return stringlike.decode('utf8')
+    else:
+        return str(stringlike)
+
 
 class Service(object):
     """
@@ -129,7 +152,7 @@ class Service(object):
     example::
 
       from intermine.webservice import Service
-      service = Service("http://www.flymine.org/query/service")
+      service = Service("https://www.flymine.org/query/service")
 
       template = service.get_template("Gene_Pathways")
       for row in template.results(A={"value":"zen"}):
@@ -179,38 +202,38 @@ class Service(object):
 
     @see: L{intermine.query}
     """
-    QUERY_PATH             = '/query/results'
-    LIST_ENRICHMENT_PATH   = '/list/enrichment'
-    WIDGETS_PATH           = '/widgets'
-    SEARCH_PATH            = '/search'
+    QUERY_PATH = '/query/results'
+    LIST_ENRICHMENT_PATH = '/list/enrichment'
+    WIDGETS_PATH = '/widgets'
+    SEARCH_PATH = '/search'
     QUERY_LIST_UPLOAD_PATH = '/query/tolist'
     QUERY_LIST_APPEND_PATH = '/query/append/tolist'
-    MODEL_PATH             = '/model'
-    TEMPLATES_PATH         = '/templates'
-    TEMPLATEQUERY_PATH     = '/template/results'
-    LIST_PATH              = '/lists'
-    LIST_CREATION_PATH     = '/lists'
-    LIST_RENAME_PATH       = '/lists/rename'
-    LIST_APPENDING_PATH    = '/lists/append'
-    LIST_TAG_PATH          = '/list/tags'
-    SAVEDQUERY_PATH        = '/savedqueries/xml'
-    VERSION_PATH           = '/version/ws'
-    RELEASE_PATH           = '/version/release'
-    SCHEME                 = 'http://'
+    MODEL_PATH = '/model'
+    TEMPLATES_PATH = '/templates'
+    TEMPLATEQUERY_PATH = '/template/results'
+    LIST_PATH = '/lists'
+    LIST_CREATION_PATH = '/lists'
+    LIST_RENAME_PATH = '/lists/rename'
+    LIST_APPENDING_PATH = '/lists/append'
+    LIST_TAG_PATH = '/list/tags'
+    SAVEDQUERY_PATH = '/savedqueries/xml'
+    VERSION_PATH = '/version/ws'
+    RELEASE_PATH = '/version/release'
+    SCHEME = 'http://'
     SERVICE_RESOLUTION_PATH = "/check/"
-    IDS_PATH               = "/ids"
-    USERS_PATH             = "/users"
+    IDS_PATH = "/ids"
+    USERS_PATH = "/users"
 
     def __init__(self, root,
-            username=None, password=None, token=None,
-            prefetch_depth=1, prefetch_id_only=False):
+                 username=None, password=None, token=None,
+                 prefetch_depth=1, prefetch_id_only=False):
         """
         Constructor
         ===========
 
         Construct a connection to a webservice::
 
-            url = "http://www.flymine.org/query/service"
+            url = "https://www.flymine.org/query/service"
 
             # An unauthenticated connection - access to all public data
             service = Service(url)
@@ -237,8 +260,10 @@ class Service(object):
 
         """
         o = urlparse(root)
-        if not o.scheme: root = "http://" + root
-        if not root.endswith("/service"): root = root + "/service"
+        if not o.scheme:
+            root = "http://" + root
+        if not root.endswith("/service"):
+            root = root + "/service"
 
         self.root = root
         self.prefetch_depth = prefetch_depth
@@ -252,13 +277,17 @@ class Service(object):
         self._list_manager = ListManager(self)
         self.__missing_method_name = None
         if token:
+            if token == "random":
+                token = self.get_anonymous_token(url=root)
             self.opener = InterMineURLOpener(token=token)
         elif username:
             if token:
-                raise ValueError("Both username and token credentials supplied")
+                raise ValueError(
+                    "Both username and token credentials supplied")
 
             if not password:
-                raise ValueError("Username given, but no password supplied")
+                raise ValueError(
+                    "Username given, but no password supplied")
 
             self.opener = InterMineURLOpener((username, password))
         else:
@@ -266,11 +295,15 @@ class Service(object):
 
         try:
             self.version
-        except WebserviceError, e:
-            raise ServiceError("Could not validate service - is the root url (%s) correct? %s" % (root, e))
+        except WebserviceError as e:
+            raise ServiceError(
+                "Could not validate service - is the root url (%s) correct? %s"
+                % (root, e))
 
         if token and self.version < 6:
-            raise ServiceError("This service does not support API access token authentication")
+            raise ServiceError(
+                "This service does not support API access token authentication"
+            )
 
         # Set up sugary aliases
         self.query = self.new_query
@@ -278,8 +311,19 @@ class Service(object):
     # Delegated list methods
 
     LIST_MANAGER_METHODS = frozenset(["get_list", "get_all_lists",
-        "get_all_list_names",
-        "create_list", "get_list_count", "delete_lists", "l"])
+                                      "get_all_list_names",
+                                      "create_list", "get_list_count",
+                                      "delete_lists", "l"])
+
+    def get_anonymous_token(self, url):
+        """
+        Generates an anonymous session token valid for 24 hours
+        =======================================================
+        """
+        url += "/session"
+        token = requests.get(url=url)
+        token = token.json()["token"]
+        return token
 
     def list_manager(self):
         """
@@ -309,7 +353,7 @@ class Service(object):
             return method
         raise AttributeError("Could not find " + name)
 
-    def __del__(self): # On going out of scope, try and clean up.
+    def __del__(self):  # On going out of scope, try and clean up.
         try:
             self._list_manager.delete_temporary_lists()
         except ReferenceError:
@@ -329,12 +373,17 @@ class Service(object):
 
         @rtype: int
         """
-        if self._version is None:
-            try:
-                url = self.root + self.VERSION_PATH
-                self._version = int(self.opener.open(url).read())
-            except ValueError, e:
-                raise ServiceError("Could not parse a valid webservice version: " + str(e))
+        try:
+            if self._version is None:
+                try:
+                    url = self.root + self.VERSION_PATH
+                    self._version = int(self.opener.open(url).read())
+                except ValueError as e:
+                    raise ServiceError(
+                        "Could not parse a valid webservice version: "
+                        + str(e))
+        except AttributeError as e:
+            raise Exception(e)
         return self._version
 
     def resolve_service_path(self, variant):
@@ -360,7 +409,8 @@ class Service(object):
         @rtype: string
         """
         if self._release is None:
-            self._release = urllib.urlopen(self.root + self.RELEASE_PATH).read()
+            self._release = ensure_str(
+                urlopen(self.root + self.RELEASE_PATH).read()).strip()
         return self._release
 
     def load_query(self, xml, root=None):
@@ -390,13 +440,20 @@ class Service(object):
         if len(columns) == 1:
             view = columns[0]
             if isinstance(view, Attribute):
-                return Query(self.model, self).select("%s.%s" % (view.declared_in.name, view))
+                return Query(self.model, self).select("%s.%s" %
+                                                      (view.declared_in.name,
+                                                       view))
+
             if isinstance(view, Reference):
-                return Query(self.model, self).select("%s.%s.*" % (view.declared_in.name, view))
+                return Query(self.model, self).select("%s.%s.*" %
+                                                      (view.declared_in.name,
+                                                       view))
             elif not isinstance(view, Column) and not str(view).endswith("*"):
                 path = self.model.make_path(view)
                 if not path.is_attribute():
-                    return Query(self.model, self).select(str(view) + ".*")
+                    return Query(self.model, self).select(
+                        str(view) + ".*")
+
         return Query(self.model, self).select(*columns)
 
     new_query = select
@@ -425,25 +482,25 @@ class Service(object):
             t = self.templates[name]
         except KeyError:
             raise ServiceError("There is no template called '"
-                + name + "' at this service")
+                               + name + "' at this service")
         if not isinstance(t, Template):
             t = Template.from_xml(t, self.model, self)
             self.templates[name] = t
         return t
 
-    def _get_json(self, path, payload = None):
-        opener = self.opener.clone()
-        opener.addheader('Accept', 'application/json')
-        with closing(opener.open(self.root + path, payload)) as resp:
-            data = json.loads(resp.read())
+    def _get_json(self, path, payload=None):
+        headers = {'Accept': 'application/json'}
+        with closing(self.opener.open(self.root + path, payload,
+                                      headers=headers)) as resp:
+            data = json.loads(ensure_str(resp.read()))
             if data['error'] is not None:
                 raise ServiceError(data['error'])
             return data
 
     def _get_xml(self, path):
-        opener = self.opener.clone()
-        opener.addheader('Accept', 'application/xml')
-        with closing(opener.open(self.root + path)) as sock:
+        headers = {'Accept': 'application/xml'}
+        with closing(self.opener.open(self.root + path,
+                                      headers=headers)) as sock:
             return minidom.parse(sock)
 
     def search(self, term, **facets):
@@ -460,15 +517,15 @@ class Service(object):
 
         @return (list, dict) The results, and a dictionary of facetting informtation.
         """
-        if isinstance(term, unicode):
+        if hasattr(term, 'encode'):
             term = term.encode('utf8')
         params = [('q', term)]
-        for facet, value in facets.iteritems():
-            if isinstance(value, unicode):
+        for facet, value in list(facets.items()):
+            if hasattr(value, 'encode'):
                 value = value.encode('utf8')
-            params.append(("facet_" + str(facet), value))
-        payload = urllib.urlencode(params, doseq = True)
-        resp = self._get_json(self.SEARCH_PATH, payload = payload)
+            params.append(("facet_{0}".format(facet), value))
+        payload = urlencode(params, doseq=True)
+        resp = self._get_json(self.SEARCH_PATH, payload=payload)
         return (resp['results'], resp['facets'])
 
     @property
@@ -490,7 +547,8 @@ class Service(object):
             self._widgets = dict(([w['name'], w] for w in ws))
         return self._widgets
 
-    def resolve_ids(self, data_type, identifiers, extra = '', case_sensitive = False, wildcards = False):
+    def resolve_ids(self, data_type, identifiers, extra='',
+                    case_sensitive=False, wildcards=False):
         """
         Submit an Identifier Resolution Job
         ===================================
@@ -516,7 +574,8 @@ class Service(object):
         @return: {idresolution.Job} The job.
         """
         if self.version < 10:
-            raise ServiceError("This feature requires API version 10+")
+            raise ServiceError(
+                "This feature requires API version 10+")
         if not data_type:
             raise ServiceError("No data-type supplied")
         if not identifiers:
@@ -529,7 +588,8 @@ class Service(object):
             "caseSensitive": case_sensitive,
             "wildCards": wildcards
         })
-        text = self.opener.post_content(self.root + self.IDS_PATH, data, InterMineURLOpener.JSON)
+        text = self.opener.post_content(
+            self.root + self.IDS_PATH, data, InterMineURLOpener.JSON)
         ret = json.loads(text)
         if ret['error'] is not None:
             raise ServiceError(ret['error'])
@@ -577,7 +637,8 @@ class Service(object):
             for e in dom.getElementsByTagName('template'):
                 name = e.getAttribute('name')
                 if name in templates:
-                    raise ServiceError("Two templates with same name: " + name)
+                    raise ServiceError(
+                        "Two templates with same name: " + name)
                 else:
                     templates[name] = e.toxml()
             self._templates = templates
@@ -628,7 +689,8 @@ class Service(object):
 
         @return: L{intermine.webservice.ResultIterator}
         """
-        return ResultIterator(self, path, params, rowformat, view, cld)
+        return ResultIterator(
+            self, path, params, rowformat, view, cld)
 
     @requires_version(9)
     def register(self, username, password):
@@ -638,22 +700,21 @@ class Service(object):
 
         @return {Service} an authenticated service.
         """
-        if isinstance(username, unicode):
-            username = username.encode('utf8')
-        if isinstance(password, unicode):
-            password = password.encode('utf8')
-        payload = urllib.urlencode({'name': username, 'password': password})
+        username = bytearray(username, 'utf8')
+        password = bytearray(password, 'utf8')
+        payload = urlencode({'name': username, 'password': password})
         registrar = Service(self.root)
-        resp = registrar._get_json(self.USERS_PATH, payload = payload)
+        resp = registrar._get_json(self.USERS_PATH, payload=payload)
         token = resp['user']['temporaryToken']
-        return Service(self.root, token = token)
+        return Service(self.root, token=token)
 
     @requires_version(16)
-    def get_deregistration_token(self, validity = 300):
+    def get_deregistration_token(self, validity=300):
         if validity < 1 or validity > 24 * 60 * 60:
-            raise ValueError("Validity not a reasonable value: 1ms - 2hrs")
-        params = urllib.urlencode({'validity': str(validity)})
-        resp = self._get_json('/user/deregistration', payload = params)
+            raise ValueError(
+                "Validity not a reasonable value: 1ms - 2hrs")
+        params = urlencode({'validity': str(validity)})
+        resp = self._get_json('/user/deregistration', payload=params)
         return resp['token']
 
     @requires_version(16)
@@ -668,13 +729,12 @@ class Service(object):
         """
         if 'uuid' in deregistration_token:
             deregistration_token = deregistration_token['uuid']
-        if isinstance(deregistration_token, unicode):
-            deregistration_token = deregistration_token.encode('utf8')
 
         path = self.root + '/user'
-        params = {'deregistrationToken': deregistration_token, 'format': 'xml'}
-        uri = path + '?' + urllib.urlencode(params)
+        params = {
+            'deregistrationToken': deregistration_token,
+            'format': 'xml'}
+        uri = path + '?' + urlencode(params)
         self.flush()
         userdata = self.opener.delete(uri)
         return userdata
-
